HELPER_FILE_NAME = 'RemoteHelper.DLL'
UPLOAD_LIST_PATH = 'Rockwell Software\\RSViewME\\Runtime\\Results.txt'

helper_path = '\\Windows'
helper_file_path = helper_path + '\\' + HELPER_FILE_NAME

storage_path = '\\Application Data'
upload_list_path = storage_path + '\\' + UPLOAD_LIST_PATH